function PlotClusters(data, clusters, windowTitle, plotTitle, subPlot, axisEqual)
% Plot 2D/3D of clustered data using several colors and symbols
% 
% PlotClusters(data, clusters, windowTitle, plotTitle, subPlot);
% 
% - Input arguments
%     data:           data to plot
%     clusters:       cluster indexes for each point in data
%     windowTitle:    title of window
%     plotTitle:      title of plot area
%     [subPlot]:      stucture containing sub plot parameters:
%                         method:       'dendrogram' or 'silhouette'
%                         parameter:    clustTree or distance variable
%     [axisEqual]:    set axis equal state (used only on reduced data)
%
%
% CLASSE Project
% Created by Edgar Ferreira - FCTUC
% edgars@student.dei.uc.pt / edgar.bajouca@gmail.com
% Updated June 15, 2007
%
% Copyright (C) 2007 Edgar Ferreira  FCTUC Coimbra, Portugal
% 
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; either version 2
% of the License, or (at your option) any later version.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
% 02110-1301, USA.


% Create figure
figure('Name', windowTitle);   % create figure

% Get maximum number of clusters
nr = round(max(clusters));

% Check subplots
if exist('subPlot','var')
    if ~isempty(subPlot)
        subplot(1,2,1);
        if strcmp(subPlot.method,'dendrogram')
            h = dendrogram(subPlot.parameter,nr);       % generate dendrogram
            title('Dendrogram Plot')
        elseif strcmp(subPlot.method,'silhouette')
            silhouette(data,clusters,subPlot.parameter);      % generate silhouette
            title('Silhouette Plot')
        end
        subplot(1,2,2);
    end
end

% Distinguish clustering plot and groups plot
name='Cluster ';
if exist('axisEqual','var')
    name='Group ';
end

% Routines to alternate colors and symbols
simb = {'+','o','*','<','.','x','s','d','v','>','p','h','^'};   % 13 symbols
cm=colormap;                                                    % 64 colors
j=1;
k=1;
l=1;
if nr<64; k=floor(64/nr); end   % color step
leg=[];                         % initialize legend

% Plot cycle
for i = 1:nr
    tempIndex = find(clusters==i);                  %return indexes for each cluster
    if ~isempty(tempIndex)
        if isempty(leg) %start legend
            leg=[name num2str(i)];         
        else            %add legend
            leg=strvcat(leg, [name num2str(i)]);
        end
        if j>64; j=1; end                           %repeat color cycle
        if l>13; l=1; end                           %repeat symbol cycle

        if size(data,2)>2
            plot3(data(tempIndex,1),data(tempIndex,2),data(tempIndex,3),simb{l},'color',cm(j,:));   %plot current cluster    
        elseif size(data,2)==2
            plot(data(tempIndex,1),data(tempIndex,2),simb{l},'color',cm(j,:));                      %plot current cluster
        end
        hold on
        j=j+k;      %increase color index
        l=l+1;      %increase symbol index
    end
end

% Labels and legend
xlabel('1st Dimension'); ylabel('2nd Dimension'); zlabel('3rd Dimension');
grid on
title(plotTitle)
legend(leg)
if exist('axisEqual','var')
    if ~isempty(axisEqual)
        if axisEqual==1
            axis equal
        end
    end
end