function clusters = SomClustering(method,data,sM,par1,par2,prot)
% SomClustering computes and plots 2D/3D clusters from SOM Map data using
% several methods and parameters
%
% clusters = SomClustering(method,data,sM,par1,par2,prot);
% 
% - Input arguments
%     method:         k-means, hierar, subt or u-matrix
%     data:           original sD.data
%     sM              map
%     par1/2:         parameters
%     prot:           prototypes selected on/off
%
% - Output arguments
%     clusters:       cluster index vector with data length
% 
% 
% CLASSE Project
% Created by Edgar Ferreira - FCTUC
% edgars@student.dei.uc.pt / edgar.bajouca@gmail.com
% Updated April 20, 2007
%
% Copyright (C) 2007 Edgar Ferreira  FCTUC Coimbra, Portugal
% 
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; either version 2
% of the License, or (at your option) any later version.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.


%% Switch methods to find codebook clusters

switch lower(method)
   case 'k-means'
      % par1: maximum clusters
      % par2: defined clusters
      tit='K-Means';
      if par1>0
          [c,p,err,ind] = kmeans_clusters(sM.codebook, par1, 20); % find at most maxi clusters
          [dummy,i] = min(ind);
          nod=p{i,1};       %nodes for lower Davies-Bouldin index
          % plot indexes and errors
          figure('name','SOM K-Means Clustering')
          plot(ind,'.-')
          hold on
          plot(err/max(err),'.-r')
          hold off
          legend('Davies-Bouldin Index','Normalized Sum of Errors')
          xlabel(['Lowest Davies-Bouldin index: ' num2str(i) ' Clusters.'])
          title(['SOM K-Means for ' num2str(par1) ' clusters'])
      elseif par2>0
          [c,p,err,ind] = kmeans_clusters(sM.codebook, par2, 20); % find at most maxi clusters
          nod=p{par2,1};    %nodes for defined number of clusters
      end
   case 'u-matrix'
      % par1: linkage
      % par2: neighbors
      tit='U-Matrix';
      nod = som_dmatclusters(sM,par1,['N' num2str(par2)],[]);
   case 'hierar'
      % par1: number of clusters
      % par2: tree method
      tit='Hierarchical';
      nod = clusterdata(sM.codebook,'distance','euclidean','linkage',par2,par1);
   case 'subt'
      % par1: radii range
      % par2: [accept  reject]
      tit='Subtractive';
      [C,S] = subclust(sM.codebook,par1,[],[1.25 par2(1) par2(2) 0]);
      % Define clusters for each point 
      temp=dist(sM.codebook,C');
      if size(temp,2)>1
          [dummy,i]=min(temp');
          nod=i';
      else
          nod=ones(size(sM.codebook,1),1);
      end
end

%% Find relation between codebook points and original data

d=dist(data,sM.codebook');
[dummy,ind]=min(d');
clusters=nod(ind);     %set clusters to original data
nr=max(clusters);
code=sM.codebook;
%nr=floor(nr);

% If data dimension is bigger than 3, find PCA projections for plotting
if size(data,2)>3
    data = pcaproj(data,3);
    code = pcaproj(sM.codebook,3);
end

%% Plot Clusters

PlotClusters(data, clusters, 'SOM Clustering Result', ['SOM ' tit ' Clustering - ' num2str(nr) ' Clusters'])

% Plot clustered prototypes
if prot==1
    som_grid(sM,'Coord',code)
    cm=colormap;
    j=1;k=1;l=1;
    if nr<64; k=floor(64/nr); end   % color step
    for i = 1:nr
        if j>64; j=1; end         %repeat color cycle
        if l>13; l=1; end         %repeat symbol cycle
        clust = find(nod==i); 
        if size(data,2)==2
            plot(code(clust,1),code(clust,2),'x','color',cm(j,:))
        elseif size(data,2)==3
            plot3(code(clust,1),code(clust,2),code(clust,3),'x','color',cm(j,:))
        end
        j=j+k;
        l=l+1;
    end
end
