function outputStruct = ClasseNormalization(rawdata)
% 
% Calculates diferent kinds of normalization and stores
% the result in a stucture.
% 
% outputStruct = ClasseNormalization(rawdata)
% 
% - Input arguments
%     rawData:        numeric data (read by xlsread)
%     
% - Output arguments
%     outputStruct:       structure with 4 fields:
%
%           original:  read data;
%           n_mean:    data with zero mean;
%           n_wiseStD: data with maximum standard deviation of 1
%           norm:      data with zero mean and standard deviation of 1
%
%
% CLASSE Project
% Created by Paulo Barbeiro - FCTUC
% barbeiro@student.dei.uc.pt / paulobarbeiro@gmail.com
% Updated April 20, 2007
%
% Copyright (C) 2007 Paulo Barbeiro  FCTUC Coimbra, Portugal
% 
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; either version 2
% of the License, or (at your option) any later version.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
% 02110-1301, USA.

%
% Each data variable (column) is normalized
%

nrOfLines=size(rawdata,1);

% Calculate zero mean data
n_mean=rawdata-repmat(mean(rawdata),nrOfLines,1);

% Calculate data with maximum Standard Deviation value of 1
stdOfColumns=std(rawdata);
indexesToChange = stdOfColumns<1;
stdOfColumns(indexesToChange)=1;
n_wiseStD=rawdata./repmat(stdOfColumns,nrOfLines,1);

% Calculate data with zero mean and Maximum
% Standrd Deviation of 1
%
% StdOfColumns and indexesToChange are re-used to save memory

stdOfColumns = std(n_mean);
indexesToChange = stdOfColumns<1;
stdOfColumns(indexesToChange)=1;
n_both=n_mean./repmat(stdOfColumns,nrOfLines,1);

% Generates Output Structure
outputStruct.original=rawdata;
outputStruct.n_mean=n_mean;
outputStruct.n_wiseStD=n_wiseStD;
outputStruct.norm=n_both;
