function [labelsStruct numData] = decodeExcelData(numData,textData)
% 
% Parses and generates text as in this scheme
%
% Header |label | label | label |...|
% -------------------------------
% legend |
% legend |
% legend |
% legend |
% legend |
% legend |
% 
%
% NOTE : if a legend Header is empty, it will assume that data is written
% in Date - Time format
% 
% labelsStruct = decodeExcelData(numData,textData)
% 
% - Input arguments
%     numData:        numeric data (read by xlsread)
%     textData:       textual data (read by xlsread)
%
% - Output arguments
%     labelsStruct:   structure with string array header(s),
%                   string array varTags (labels), and with
%                   n string arrays named 'Label_n' for each
%                   column of legend
%
%
% CLASSE Project
% Created by Paulo Barbeiro - FCTUC
% barbeiro@student.dei.uc.pt / paulobarbeiro@gmail.com
% Updated April 20, 2007
%
% Copyright (C) 2007 Paulo Barbeiro  FCTUC Coimbra, Portugal
% 
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; either version 2
% of the License, or (at your option) any later version.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
% 02110-1301, USA.

    lastColOfNans=0;
    [numDataRows,numDataCols]=size(numData);
    [textDataRows,textDataCols]=size(textData);
    
    for i=1:numDataCols
        if sum(isnan(numData(:,i)))==numDataRows
            lastColOfNans=i;
        end 
    end
    for i=1:lastColOfNans-1
        for j= 1+textDataRows-numDataRows:textDataRows
            textData(j,i)={num2str(numData(j-(textDataRows-numDataRows),i))};
        end
    end
    numData(:,1:lastColOfNans,:)=[];
    
    [numDataRows,numDataCols]=size(numData);
    [textDataRows,textDataCols]=size(textData);    
    clear cont l c
    
    
    
    if(textDataCols==0)
    %there are no labels
        eventLabels=labelsGenerator('Event ',numDataRows);
        header = {'# Event'};
        varsTags = labelsGenerator('Variable ',numDataCols);
    elseif(textDataCols==numDataCols)
    % if there are only Variables Tags..
        numberOfExtraTextRows = textDataRows-numDataRows;
        eventLabels=labelsGenerator('Event ',numDataRows);
        header = {'# Event'};
        if (textDataRows>1)
            varsTags=cellsToSingleRow(textData,1);
        else
            varsTags=textData;
        end
    elseif(textDataRows==numDataRows)
    % if there are only event labels..
        numberOfExtraTextRows=0;
        varsTags = labelsGenerator('Variable',numDataCols);
        header = labelsGenerator('# Event',size(textData,2));
    else
    % both labels exist
        numberOfExtraTextRows = textDataRows-numDataRows;
        numberOfExtraTextCols = textDataCols-numDataCols;
        
        if (numberOfExtraTextRows>1)
            varsTags=cellsToSingleRow(textData(1:numberOfExtraTextRows,(numberOfExtraTextCols+1):end),false);
            header=cellsToSingleRow(textData(1:numberOfExtraTextRows,1:numberOfExtraTextCols),false);
        else
            varsTags = textData(1,(numberOfExtraTextCols+1):end);
            header = textData(1,1:numberOfExtraTextCols);
        end
    end


    % if there are events
    
    if(~exist('eventLabels','var'))
        for i=1 : length(header)
            if strcmp(char(header(i)),char({''}))
                labelsStruct.(['Label_' num2str(i)]) = reWrite_DateTime(textData(numberOfExtraTextRows + 1:end,i));
                header(i) = {'Date / Time'};
            else
                labelsStruct.(['Label_' num2str(i)]) = textData(numberOfExtraTextRows + 1:end,i);
            end
        end
    else
        labelsStruct.('Label_1')=eventLabels;
    end

    labelsStruct.varNames = varsTags;
    labelsStruct.header = header;
    
end
%% Generates Sequencial Labels from text
function lb = labelsGenerator(text,n)

    lb = cell(1,n);
    for i=1 : n
        lb(i)={[text num2str(i)]};
    end
end

%% Converts cell Arrays to single Row
function singleRow = cellsToSingleRow(inputCells,showEmpties)
    singleRow = cell(1,size(inputCells,2));
    for j=1 : size(inputCells,2)
        temp=char(inputCells(1,j));
        for i=2 : size(inputCells,1)
            if (showEmpties)
                temp = [temp  ' / ' char(inputCells(i,j))];
            else
                if(~strcmp(inputCells(i,j),''))
                    if(strcmp(temp,char({''})))
                        temp = char(inputCells(i,j));
                    else
                        temp = [temp ' / ' char(inputCells(i,j))];
                    end
                end
            end
        end
        singleRow(j)={temp};
    end
end

%% In the case of time events, an automatic text correction is made fo
% some info that is lost during exel-matlab conversion of data
function out =reWrite_DateTime(in)

    in=char(in);
    maxlength=size(in,2);
    out=cell(size(in,1),1);
    for i=1 : size(in,1)
        if(strcmp(in(i,12:min(18,maxlength)),'        ')==1 || strcmp(in(i,12:min(18,maxlength)),'       ')==1)
            in(i,12:min(18,maxlength))='0:00:00';
        end
        if(strcmp(in(i,min(19,maxlength)),' ')==1)
            in(i,12:min(19,maxlength))=strcat('0',in(i,12:18));
        end
        out(i)={in(i,:)};
    end
end

