function [data_available,handles,info2] = CheckDataAvailability(handles,needOutput)
% Check if input and output data are available
% 
% CheckDataAvailability(handles,needOutput);
% 
% - Input arguments
%     handles:          handles to GUI objects and data
%     needOutput:       whether output data should be checked for availability
%
% - Output arguments
%     data_available:   whether the needed data is available
%     handles:          updated handles
%     info2:            information for user about the data
%
% CLASSE Project
% Created by Sara Silva - FCTUC
% sara@dei.uc.pt
% Updated September 21, 2008
%
% Copyright (C) 2008 Sara Silva  FCTUC Coimbra, Portugal
% 
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; either version 2
% of the License, or (at your option) any later version.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
% 02110-1301, USA.



data_available=1;

% Determine the selected input data set
      instr = get(handles.learn_input_data_popupmenu,'String');
      inval = get(handles.learn_input_data_popupmenu,'Value');
      % Set current learning input data to the selected data set.
      switch instr{inval};
        case 'Raw'
            % check if raw data is available:
            if isfield(handles,'current_data')
                handles.learningData.input_data=handles.current_data;
                info2=['  ' instr{inval} ' Input'];
            else
                data_available=0;
                warndlg('Raw input data is not currently available','Data unavailable')
            end
        case 'Reduced'
            % check if reduced data is available:
            if isfield(handles,'reducedData')
                if isfield(handles.reducedData,'dataMatrix')
                    strdims = get(handles.popupmenu_learn_dims,'String');
                    valdims = get(handles.popupmenu_learn_dims,'Value');
                    ndims=str2double(strdims{valdims});
                    if ndims>size(handles.reducedData.dataMatrix,2)
                        warndlg('Reduced input data has lower dimension than selected','Data unavailable')
                    else
                        handles.learningData.input_data=handles.reducedData.dataMatrix(:,1:ndims);
                    end
                    info2=['  ' instr{inval} ' Input (' int2str(ndims) ' dimensions)'];
                end
            else
                data_available=0;
                warndlg('Reduced data is not currently available','Data unavailable')
            end
      end

% Determine the selected output data set
if needOutput || isfield(handles.labels,'groups')
      if isfield(handles.labels,'groups')
          if strcmp(get(handles.svm_learn_output_data_popupmenu,'Visible'),'on') %SVM
              handles.learningData.output_data=handles.labels.groups.original;
              if data_available
                info2=[info2 ' and SVM Modified Output.'];
              else
                info2='';
              end
          else
            outstr = get(handles.learn_output_data_popupmenu,'String');
            outval = get(handles.learn_output_data_popupmenu,'Value');
            % Set current learning input data to the selected data set.
            switch outstr{outval};
                case 'Raw' 
                    handles.learningData.output_data=handles.labels.groups.original;
                case 'Binary' 
                    handles.learningData.output_data=handles.labels.groups.binary;
                case 'Bipolar' 
                    handles.learningData.output_data=handles.labels.groups.bipolar;
            end
            if data_available
                info2=[info2 ' and ' outstr{outval} ' Output.'];
            else
                info2='';
            end
          end
      else
          data_available=0;
          warndlg('Expected output data is not available','Data unavailable')
      end
end