function outputStruct = ClasseCodification(rawdata)
% 
% Calculates diferent kinds of codification and stores
% the result in a stucture.
% 
% outputStruct = ClasseCodification(rawdata)
% 
% - Input arguments
%     rawData:        numeric data (read by xlsread)
%     
% - Output arguments
%     outputStruct:       structure with 3 fields:
%
%           original:  read data (integer);
%           binary:    data represented as binary outputs;
%                      (examples: 2 = [0 1 0 0], 4 = [0 0 0 1])
%           bipolar:   data represented as bipolar outputs;
%                      (example: 3 = [-1 -1 +1 -1])
%
%
% CLASSE Project
% Created by Sara Silva - FCTUC (based on ClasseNormalization.m)
% sara@dei.uc.pt
% Updated July 1, 2008
%
% Copyright (C) 2008 Sara Silva  FCTUC Coimbra, Portugal
% 
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; either version 2
% of the License, or (at your option) any later version.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
% 02110-1301, USA.

%
% The data is supposed to contain only one column of integers
%

nrOfLines=size(rawdata,1);
maxValue=max(rawdata);

% Calculate binary output:
binary=zeros(nrOfLines,maxValue);
for i=1:nrOfLines
    binary(i,rawdata(i))=1;
end

% Calculate bipolar output:
bipolar=ones(nrOfLines,maxValue);
bipolar(binary==0)=-1;

% Generates Output Structure
outputStruct.original=rawdata;
outputStruct.binary=binary;
outputStruct.bipolar=bipolar;
