function classification = ClasseDecodification(nnoutput)
% 
% Returns a classification from the output of a neural network.
% 
% classification = ClasseDecodification(nnoutput)
% 
% - Input arguments
%     nnoutput:        numeric data (given by a neural network)
%     
% - Output arguments
%     classification:  numeric identification of classes, given by
%                      the maximum value of each line
%
%     example:         nnoutput = [ 0.2 0.9  1.05 0.1
%                                  -1.6 0.4 -1.05 0.9]
%
%                      classification = [3 ; 4]
%
%
% CLASSE Project
% Created by Sara Silva - FCTUC
% sara@dei.uc.pt
% Updated July 1, 2008
%
% Copyright (C) 2008 Sara Silva  FCTUC Coimbra, Portugal
% 
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; either version 2
% of the License, or (at your option) any later version.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
% 02110-1301, USA.


nrOfCols=size(nnoutput,2);

if nrOfCols==1
    
    % if only one column, return closest integer
    classification=round(nnoutput);
    
else
    
    % ans = row indicator, we don't need it
    % classification = column indicator, specifies the maximum of the row
    [r,c]=find(nnoutput==repmat(max(nnoutput,[],2),1,nrOfCols));
    rc=sortrows([r c],1);
    classification=rc(:,2);
    
end