function [classification,n_unclassified]=ClasseSVMclassify(input_data,output_data,nn)
% 
% Returns a n-class classification from several trained SVMs.
% 
% classification = ClasseSVMclassify(input_data,output_data,nn)
% 
% - Input arguments
%     input_data:        input data (matrix) to classify
%     output_data:       expected classification (vector)
%     nn:                cell array of trained SVMs
%     
% - Output arguments
%     classification:    vector of obtained classification
%
% CLASSE Project
% Created by Sara Silva - FCTUC
% sara@dei.uc.pt
% Updated September 20, 2008
%
% Copyright (C) 2008 Sara Silva  FCTUC Coimbra, Portugal
% 
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; either version 2
% of the License, or (at your option) any later version.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
% 02110-1301, USA.

classification=[];
n_unclassified=0;

% how many output classes?
unique_classes=unique(output_data);
nclasses=length(unique_classes);

    % for each of them, use a trained network:
    for i=1:nclasses
        this_class=find(output_data==unique_classes(i));
        not_this_class=find(output_data~=unique_classes(i));
        % modify training output to only 1 and -1 values:
        modified_output_data=output_data;
        modified_output_data(this_class)=1;
        modified_output_data(not_this_class)=-1;
        % use a trained svm:
        [err,predictions(i,:)]=svmclassify(input_data,modified_output_data,nn{i}.nn);
    end

    % get maximum predicted values for each sample:
    max_values=max(predictions,[],1);

    % for each sample, pick class corresponding to SVM with max prediction:
    for i=1:length(predictions)
        f=find(predictions(:,i)==max_values(i));
        if length(f)==1
            classification(i)=nn{f}.class;
        else
            n_unclassified=n_unclassified+1;
        end
    end
    
    classification=classification';

