function nn=ClasseSVMlearn(input_data,output_data,params)
% 
% Trains as many SVMs as needed to perform n-class classification.
% 
% nn = ClasseSVMlearn(input_data,output_data,params)
% 
% - Input arguments
%     input_data:        training input matrix
%     output_data:       training output vector
%     params:            string of parameters to use in training
%     
% - Output arguments
%     nn:       cell array with 2 elements:
%               - nn: trained SVM
%               - class: which class the network corresponds to
%
% CLASSE Project
% Created by Sara Silva - FCTUC
% sara@dei.uc.pt
% Updated September 20, 2008
%
% Copyright (C) 2008 Sara Silva  FCTUC Coimbra, Portugal
% 
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; either version 2
% of the License, or (at your option) any later version.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
% 02110-1301, USA.


% how many output classes?
unique_classes=unique(output_data);
nclasses=length(unique_classes);

% for each of them, train a network:
for i=1:nclasses
   this_class=find(output_data==unique_classes(i));
   not_this_class=find(output_data~=unique_classes(i));
   % modify training output to only 1 and -1 values:
   modified_output_data=output_data;
   modified_output_data(this_class)=1;
   modified_output_data(not_this_class)=-1;
   % train a svm:
   nn{i}.nn=svmlearn(input_data,modified_output_data,params);
   % store which class this corresponds to:
   nn{i}.class=unique_classes(i);
end
