function PlotClassification(expected,obtained,windowTitle,plotTitle)
% Plot expected vs obtained classification
% 
% PlotClassification(expected,obtained);
% 
% - Input arguments
%     expected:       expected classification (or empty [])
%     obtained:       structure containing obtained classification:
%                       raw:     raw nn output
%                       classes: classification calculated from raw output
%     windowTitle:    title of window
%     plotTitle:      title of plot area
%
% CLASSE Project
% Created by Sara Silva - FCTUC
% sara@dei.uc.pt
% Updated July 19, 2008
%
% Copyright (C) 2008 Sara Silva  FCTUC Coimbra, Portugal
% 
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; either version 2
% of the License, or (at your option) any later version.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
% 02110-1301, USA.


% Create figure
figure('Name', windowTitle);   % create figure

% expected classification:
if ~isempty(expected)
    plot(expected,'ko');
    leg=['Expected Class'];
    hold on
else
    leg=[];
end

% obtained raw nn output:
if size(obtained.raw,2)==1
    plot(obtained.raw,'bx');
    leg=strvcat(leg,'Obtained Raw Output');
    hold on
end

% obtained classification:
if ~isempty(expected)
    plot(obtained.classes,'g.');
    leg=strvcat(leg,'Obtained Classification (correct)');
else
    plot(obtained.classes,'c.');
    leg=strvcat(leg,'Obtained Classification');
end
hold on

% wrongly classified points:
if ~isempty(expected)
    for i=1:length(obtained.classes)
        if obtained.classes(i)~=expected(i)
            plot(i,obtained.classes(i),'r.');
            hold on
        end
    end
    leg=strvcat(leg,'Obtained Classification (incorrect)');
end

% Labels and legend
xlabel('Data points'); ylabel('Classification');
title(plotTitle)
legend(leg,'Location','Best')
