function v=ValidParameters(parameters)
% ValidParameters validates several learning and Dim.Reduction parameters
%
% v = ValidParameters(parameters);
%
% - Input arguments
%     parameters:     structure containing several parameters
%
% - Output arguments
%     v:       whether all the parameters are valid
%
%
% CLASSE Project
% Created by Sara Silva - FCTUC
% sara@dei.uc.pt
% Updated August 14, 2008
%
% Copyright (C) 2008 Sara Silva  FCTUC Coimbra, Portugal
% 
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; either version 2
% of the License, or (at your option) any later version.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
% 02110-1301, USA.


v=1;
p=fieldnames(parameters);

for i=1:length(p)
    switch p{i}
        % Dimensionality Reduction parameters
        case 'nneighbors'
            if isequalwithequalnans(parameters.nneighbors,NaN)
                errordlg('Number of nearest neighbors must be numeric.','Invalid Parameter')
                v=0;
            elseif parameters.nneighbors<=0
                errordlg('Number of nearest neighbors must be positive.','Invalid Parameter')
                v=0;
            elseif round(parameters.nneighbors)~=parameters.nneighbors
                errordlg('Number of nearest neighbors must be integer.','Invalid Parameter')
                v=0;
            end
        case 'perplexity'
            if isequalwithequalnans(parameters.perplexity,NaN)
                errordlg('Perplexity must be numeric.','Invalid Parameter')
                v=0;
            end
        case 'sigma'
            if isequalwithequalnans(parameters.sigma,NaN)
                errordlg('Sigma must be numeric.','Invalid Parameter')
                v=0;
            end
        % Learning Parameters
        case 'spread'
            if isequalwithequalnans(parameters.spread,NaN)
                errordlg('Spread must be numeric.','Invalid Parameter')
                v=0;
            elseif parameters.spread<0
                errordlg('Spread must be non negative.','Invalid Parameter')
                v=0;
            end
        case 'nneurons'
            if isequalwithequalnans(parameters.nneurons,NaN)
                errordlg('Number of hidden neurons must be numeric.','Invalid Parameter')
                v=0;
            elseif parameters.nneurons<=0
                errordlg('Number of hidden neurons must be positive.','Invalid Parameter')
                v=0;
            elseif round(parameters.nneurons)~=parameters.nneurons
                errordlg('Number of hidden neurons must be integer.','Invalid Parameter')
                v=0;
            end
        case 'initial_weight_change'
            if isequalwithequalnans(parameters.initial_weight_change,NaN)
                errordlg('Initial weight change must be numeric.','Invalid Parameter')
                v=0;
            elseif parameters.initial_weight_change<=0
                errordlg('Initial weight change must be positive.','Invalid Parameter')
                v=0;
            end    
        case 'initial_mu'
            if isequalwithequalnans(parameters.initial_mu,NaN)
                errordlg('Initial mu must be numeric.','Invalid Parameter')
                v=0;
            elseif parameters.initial_mu<=0
                errordlg('Initial mu must be positive.','Invalid Parameter')
                v=0;
            end
        case 'learning_rate'
            if isequalwithequalnans(parameters.learning_rate,NaN)
                errordlg('Learning rate must be numeric.','Invalid Parameter')
                v=0;
            elseif parameters.learning_rate<0
                errordlg('Learning rate must be non negative.','Invalid Parameter')
                v=0;
            end
         case 'momentum'
            if isequalwithequalnans(parameters.momentum,NaN)
                errordlg('Momentum must be numeric.','Invalid Parameter')
                v=0;
            elseif parameters.momentum<0 || parameters.momentum>1
                errordlg('Momentum must be a real value between 0 and 1.','Invalid Parameter')
                v=0;
            end
        case 'goal'
            if isequalwithequalnans(parameters.goal,NaN)
                errordlg('Maximum MSE must be numeric.','Invalid Parameter')
                v=0;
            elseif parameters.goal<0
                errordlg('Maximum MSE must be non negative.','Invalid Parameter')
                v=0;
            end
        case 'epochs'
            if isequalwithequalnans(parameters.epochs,NaN)
                errordlg('Maximum number of epochs must be numeric.','Invalid Parameter')
                v=0;
            elseif parameters.epochs<=0
                errordlg('Maximum number of epochs must be positive.','Invalid Parameter')
                v=0;
            elseif round(parameters.epochs)~=parameters.epochs
                errordlg('Maximum number of epochs must be integer.','Invalid Parameter')
                v=0;
            end
    end
end